// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | info@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component to display the data contained in the atmospheric model used in calculations
// Ground altitude (horizontal path values) will be applied to the current value of temperature, total pressure. pathlength, and partial pressures
import QtQuick 2.4
import QtQuick.Controls 1.3
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1
import Plot 1.0

Item {

    id: atmModelItem
    implicitHeight: atmModelItem.height * 0.6
    implicitWidth: atmModelItem.width * 0.7
    anchors.fill: parent

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: "black"
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: borderRect
        anchors.right: atmModelColumnLayout.right
        anchors.top: atmModelColumnLayout.top
        anchors.left: atmModelColumnLayout.left
        anchors.bottom: atmModelColumnLayout.bottom
        anchors.margins: commentLabel.height * (-0.85)
        color: "white"
        border.width: 1
        border.color: "black"
        radius: height * 0.02
    }

    // Rectangle to draw the form border around the control elements
    Rectangle {
        id: controlElementsBorderRect
        anchors.right: controlElementsRowLayout.right
        anchors.top: controlElementsRowLayout.top
        anchors.left: controlElementsRowLayout.left
        anchors.bottom: controlElementsRowLayout.bottom
        anchors.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? commentLabel.height * (-0.2125) : -5
        anchors.leftMargin: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? commentLabel.height * (-0.425) : -10
        color: "white"
        border.width: 1
        border.color: "black"
        radius: width * 0.02
    }

    // Column layout to hold all the plot and other interface items
    ColumnLayout {

        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: atmModelColumnLayout
        anchors.top: controlElementsBorderRect.bottom
        anchors.bottom: atmModelItem.bottom
        anchors.left: atmModelItem.left
        anchors.right: atmModelItem.right
        anchors.leftMargin: commentLabel.height * (0.935)   // atmModelItem.width * 0.05
        anchors.rightMargin: commentLabel.height * (0.935)  // atmModelItem.width * 0.05
        anchors.topMargin: commentLabel.height * (0.935)    // atmModelItem.height * 0.05
        anchors.bottomMargin: commentLabel.height * (0.935) // atmModelItem.height * 0.05
        Layout.fillWidth: true
        spacing: 5
        transformOrigin: Item.Center

        PlatformLabel {
            Layout.fillWidth: true
            horizontalAlignment: Text.AlignHCenter
            id: commentLabel
            text: qsTr("Currently only ground altitude values are used.")
        }

        // Rectangle with the plot of the atmospehric model
        Rectangle {
            id: plotRect
            anchors.right: atmModelColumnLayout.right
            anchors.top: commentLabel.bottom
            anchors.bottom: atmModelColumnLayout.bottom
            anchors.left: atmModelColumnLayout.left
            Layout.fillHeight: true
            color: "white"
            border.width: 1
            border.color: "white"
            radius: 3

            Plot {
                id: atmModel1Plot
                anchors.fill: parent
                spectrum: atmDataObj.spectrum  // Updating the spectrum when the atmospehric model selected has been changed
                minX: atmDataObj.minX
                maxX: atmDataObj.maxX
                minY: atmDataObj.minY
                maxY: atmDataObj.maxY
                xMinPadding: commentLabel.height * 2.6
                xMaxPadding: atmListView.width * 1.05
                yMinPadding: commentLabel.height * 0.9
                yMaxPadding: commentLabel.height * 2.4
                yAxisTitle: 'Altitude, km'
                xAxisTitle: atmDataObj.xAxisTitle
                lineColor: atmDataObj.lineColor
                lineThickness: 3
                numMajorXTicks: 4
                numMajorYTicks: 4
                numMinorXTicks: 4
                numMinorYTicks: 4
                plotInverted: true
                currSpecIndex: atmDataObj.currSpecIndex
            }
        }

        Label {
            id: widthLabel
            text: "ClONO2"
            visible: false
            horizontalAlignment: Text.AlignRight
        }

        // Flickable is used to add a scroll bar when the number of items is larger than the screen can fit
        ScrollView {
            id: molFlickable
            anchors.top: commentLabel.bottom
            anchors.topMargin: 5
            anchors.bottomMargin: commentLabel.height * 2.3
            anchors.right: atmModelColumnLayout.right
            anchors.bottom: atmModelColumnLayout.bottom
            implicitWidth: widthLabel.width * 1.35
            verticalScrollBarPolicy: Qt.ScrollBarAlwaysOff

            ListView {
                id: atmListView
                anchors.fill: parent
                spacing: 0
                currentIndex: atmDataObj.selectedItem
                // Need to disable the highligh animation

                model: atmDataObj

                delegate: RowLayout {

                    id: moleculeDelegate
                    implicitHeight:  molNameTextLabel.height * 1.4

                    Text {
                        id: molNameTextLabel
                        text: model.itemName + "                    "
                        horizontalAlignment: Text.AlignRight
                        verticalAlignment: Text.AlignVCenter
                    }

                    // Mouse area to capture the selection event for a particular item in a list
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            atmListView.currentIndex = index;
                            atmDataObj.selectedItem = index;
                        }
                    }
                }

                highlight:
                    Rectangle {
                        color: '#cccccc'
                    }
            }            
        }
    }

    Text {
        id: widthText
        text: "US Standard (default)"
        visible: false
        enabled: false
    }

    RowLayout {

        id: controlElementsRowLayout
        anchors.right: atmModelItem.right
        anchors.top: atmModelItem.top
        anchors.rightMargin: 5
        anchors.topMargin: 5

        // Button to close the form
        ControlButton {
            id: applyValuesButton
            platformText: qsTr("Apply")

            // Enabling the closing of the form when the exit button is pressed
            MouseArea {
                anchors.fill: parent
                onClicked: {

                    if ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows'))
                    {
                        var component = Qt.createComponent("AtmosphericModelApply.qml")
                        if (component.status === Component.Ready)
                        {
                            atmModelApplyWindow = component.createObject(atmModelItem)
                        }
                        else
                        {
                            messageDialog.show(component.errorString())
                        }
                        atmModelApplyWindow.show()
                    }
                    else
                    {
                        applyDialog.show();
                    }
                }
            }
        }

        // ComboBox to select the pathlength units
        PlatformComboBox {
            id: atmModelComboBox
            Layout.fillWidth: true
            model: [ "Tropical", "Midlatitude summer", "Midlatitude winter", "Subarctic summer", "Subarctic winter", "US Standard (default)" ]
            currentIndex: atmDataObj.modelNum
            onCurrentIndexChanged:
            {
                atmDataObj.modelNum = currentIndex
            }
            implicitWidth: (Qt.platform.os == 'ios') ? widthText.width * 1.1 : widthText.width * 1.32
            style: ((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? winReferenceComboBox.style : referenceComboBox.style
        }
        // This weird way was introduced to enable text black color under the windows phone platform while maintaining the style a default for other platforms
        PlatformComboBox {
            id: referenceComboBox
            visible: false
        }
        PlatformComboBox {
            id: winReferenceComboBox
            style: ComboBoxPlatformStyle {}
            visible: false
        }

        // Button to close the form
        ControlButton {
            id: exitButton
            platformText: qsTr("X")
            opacity: 1.0

            // Enabling the closing of the form when the exit button is pressed
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    atmModelItem.destroy();
                }
            }
        }
    }

    // Dialog to exit an applicaion
    MessageDialog {
        id: applyDialog
        icon: StandardIcon.Question
        standardButtons: StandardButton.Yes | StandardButton.No

        function show() {
            applyDialog.text = "The values in the selected atmospheric model at ground altitude (total pressure, temperature, and partial pressures) will be overwritten into the horizontal path calculation variables. Would you like to continue?";
            applyDialog.open()
        }
        onYes: {
            atmDataObj.assignNewValues();  // Reading in the atmospehric model values for the horizontal path for the current atmospheric model
            MolListViewModel.updatePartPress();  // Need to also update the values displayed in the molecules list (partial pressures)
        }
        onNo: { applyDialog.close(); }
    }
}
