// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | info@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// Hitran download options QML component (used to specify how the HITRAN database is to be accessed: 0 - segmented download as new regions are requested and saving for later reuse, 1 - complete download (auto or manual) once and save for offline use, 2 - download every time (no saving of downloaded data))
import QtQuick 2.0
import QtQuick.Controls 1.3
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1
import QtQuick.Controls.Styles 1.4

Item {

    id: hitranDownloadOptionsItem
    anchors.fill: parent

    onHeightChanged:
    {
        if (coreObj.externalStorageAccessGranted == true)  // The case when the HITRAN database cannot be stored locally
        {
            if (hitranDownloadOptionsDataObj.hitranFileUsage == 0)
            {
                option1SegmentedDownloadAndStoreCheckBox.checked = true
                option2CompleteDownloadAndStoreCheckBox.checked = false
                option3DownloadEveryTimeCheckBox.checked = false
            }
            else if (hitranDownloadOptionsDataObj.hitranFileUsage == 1)
            {
                option1SegmentedDownloadAndStoreCheckBox.checked = false
                option2CompleteDownloadAndStoreCheckBox.checked = true
                option3DownloadEveryTimeCheckBox.checked = false
            }
            else if (hitranDownloadOptionsDataObj.hitranFileUsage == 2)
            {
                option1SegmentedDownloadAndStoreCheckBox.checked = false
                option2CompleteDownloadAndStoreCheckBox.checked = false
                option3DownloadEveryTimeCheckBox.checked = true
            }
        }
        else
        {
            option1SegmentedDownloadAndStoreCheckBox.checked = false
            option2CompleteDownloadAndStoreCheckBox.checked = false
            option3DownloadEveryTimeCheckBox.checked = true
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: "black"
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: formRect
        anchors.right: hitranDownloadOptionsColumnLayout.right
        anchors.top: hitranDownloadOptionsColumnLayout.top
        anchors.left: hitranDownloadOptionsColumnLayout.left
        anchors.bottom: hitranDownloadOptionsColumnLayout.bottom
        anchors.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? commentLabel.height * (-0.85) : -20
        color: "white"
        border.width: 1
        border.color: "black"
        radius: height * 0.03
    }

    // Button to close the form
    BackRectangle {
        anchors.fill: exitButton
        anchors.margins: 8
    }
    ControlButton {
        anchors.right: hitranDownloadOptionsItem.right
        anchors.top: hitranDownloadOptionsItem.top
        id: exitButton
        platformText: qsTr("X")
        opacity: 1.0

        // Enabling the closing of the form when the exit button is pressed
        MouseArea {
            anchors.fill: parent
            onClicked: {
                hitranDownloadOptionsItem.destroy();
            }
        }
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: hitranDownloadOptionsColumnLayout
        anchors.centerIn: parent
        spacing: 5
        transformOrigin: Item.Center
        Layout.fillWidth: true
        Layout.fillHeight: true

        PlatformLabel {
            Layout.fillWidth: true
            id: commentLabel
            text: qsTr("HITRAN spectral database access options")
        }

        Rectangle {
            id: separatorRect
            Layout.fillWidth: true
            height: 1
            Layout.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows')) ? 1 : 5
            color: "black"
        }

        PlatformCheckBox {
            id: option1SegmentedDownloadAndStoreCheckBox
            onCheckedChanged: {
                if (checked)
                {
                    option1SegmentedDownloadAndStoreCheckBox.checked = true
                    option2CompleteDownloadAndStoreCheckBox.checked = false
                    option3DownloadEveryTimeCheckBox.checked = false
                    hitranDownloadOptionsDataObj.hitranFileUsage = 0;

                    // This first condition takes into account the case when the read/write access to external storage is unavailable (possible if Android Version 6.0 Marshmallow is used)
                    if (coreObj.externalStorageAccessGranted == true)
                    {
                        separator2Rect.visible = false
                        detailsLabel.visible = false
                        downloadHitranButton.visible = false
                        separator2Rect.enabled = false
                        detailsLabel.enabled = false
                        downloadHitranButton.enabled = false
                    }
                    else
                    {
                        separator2Rect.visible = true
                        detailsLabel.visible = true
                        downloadHitranButton.visible = false
                        separator2Rect.enabled = true
                        detailsLabel.enabled = true
                        downloadHitranButton.enabled = false
                    }
                }
            }
            onClicked: option1SegmentedDownloadAndStoreCheckBox.checked = true
            platformText: (Qt.platform.os == 'ios') ? qsTr("Segmented download as new regions\nare requested and saving for later reuse") : qsTr("   Segmented download as new regions\n   are requested and saving for later reuse")
            visible: coreObj.externalStorageAccessGranted ? true : false
            style: !((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? referenceCheckBox.style : 0
        }
        // This weird way was introduced to enable text black color under the windows phone platform while maintaining the style a default for other platforms
        CheckBox {
            id: referenceCheckBox
            visible: false
        }

        PlatformCheckBox {
            id: option2CompleteDownloadAndStoreCheckBox
            onCheckedChanged: {
                if (checked)
                {
                    option1SegmentedDownloadAndStoreCheckBox.checked = false
                    option2CompleteDownloadAndStoreCheckBox.checked = true
                    option3DownloadEveryTimeCheckBox.checked = false
                    hitranDownloadOptionsDataObj.hitranFileUsage = 1;

                    // This first condition takes into account the case when the read/write access to external storage is unavailable (possible if Android Version 6.0 Marshmallow is used)
                    if (coreObj.externalStorageAccessGranted == true)
                    {
                        if (hitranDownloadOptionsDataObj.cumulativeHitranExists === false)
                        {
                            separator2Rect.visible = true
                            detailsLabel.visible = true
                            downloadHitranButton.visible = true
                            separator2Rect.enabled = true
                            detailsLabel.enabled = true
                            downloadHitranButton.enabled = true
                            formRect.radius = formRect.height * 0.02
                        }
                        else
                        {
                            separator2Rect.visible = true
                            detailsLabel.visible = true
                            separator2Rect.enabled = true
                            detailsLabel.enabled = true
                            downloadHitranButton.visible = false
                            downloadHitranButton.enabled = false
                            formRect.radius = formRect.height * 0.02
                        }
                    }
                    else
                    {
                        separator2Rect.visible = true
                        detailsLabel.visible = true
                        separator2Rect.enabled = true
                        detailsLabel.enabled = true
                        downloadHitranButton.visible = false
                        downloadHitranButton.enabled = false
                        formRect.radius = formRect.height * 0.02
                    }
                }
            }
            onClicked: option2CompleteDownloadAndStoreCheckBox.checked = true
            platformText: (Qt.platform.os == 'ios') ? qsTr("Complete download once (auto or\nmanual) and offline reuse later") : qsTr("   Complete download once (auto or\n   manual) and offline reuse later")
            visible: coreObj.externalStorageAccessGranted ? true : false
            style: !((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? referenceCheckBox.style : 0
        }

        PlatformCheckBox {
            id: option3DownloadEveryTimeCheckBox            
            onCheckedChanged: {
                if (checked)
                {
                    option1SegmentedDownloadAndStoreCheckBox.checked = false
                    option2CompleteDownloadAndStoreCheckBox.checked = false
                    option3DownloadEveryTimeCheckBox.checked = true
                    hitranDownloadOptionsDataObj.hitranFileUsage = 2;

                    // This first condition takes into account the case when the read/write access to external storage is unavailable (possible if Android Version 6.0 Marshmallow is used)
                    if (coreObj.externalStorageAccessGranted == true)
                    {
                        separator2Rect.visible = false
                        detailsLabel.visible = false
                        downloadHitranButton.visible = false
                        separator2Rect.enabled = false
                        detailsLabel.enabled = false
                        downloadHitranButton.enabled = false
                    }
                    else
                    {
                        separator2Rect.visible = true
                        detailsLabel.visible = true
                        downloadHitranButton.visible = false
                        separator2Rect.enabled = true
                        detailsLabel.enabled = true
                        downloadHitranButton.enabled = false
                    }
                }
            }
            onClicked: option3DownloadEveryTimeCheckBox.checked = true
            platformText: (Qt.platform.os == 'ios') ? qsTr("Download every time\n(no saving of downloaded data ever)") : qsTr("   Download every time\n   (no saving of downloaded data ever)")
            style: !((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? referenceCheckBox.style : 0
        }

        Rectangle {
            id: separator2Rect
            Layout.fillWidth: true
            height: 1
            Layout.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows')) ? 2 : 10
            color: "black"
        }

        PlatformLabel {
            // anchors.fill: parent
            id: detailsLabel
            text: {
                if (coreObj.externalStorageAccessGranted)
                {
                    if (hitranDownloadOptionsDataObj.cumulativeHitranExists)
                    {
                        return qsTr("The complete HITRAN database is present\nunder 'Documents/bytran'.")
                    }
                    else
                    {
                        // Currently access to the Downloads and Documents folders is only available under the Android operating system.  iOS does not have such folders and the Windows platform would require C# code inserts which I am unfamiliar with
                        if ((Qt.platform.os == 'winphone') || (Qt.platform.os == 'ios'))
                        {
                            return qsTr("Offline mode requires the complete HITRAN\ndatabase to be present (~700 megabytes). \nIt may be downloaded from hitran.org.")
                        }
                        else
                        {
                            return qsTr("Offline mode requires the complete HITRAN\ndatabase to be present (~700 megabytes). \nIt may be downloaded or manually placed\ninto the 'Downloads/bytran' folder.")
                        }
                    }
                }
                else
                {
                    return qsTr("The applcation does not have permission to\nread/write to the external storage. As such\nthe only option available is to download the\nHITRAN data from hitran.org before every\ncalculation without the capability to save\nthe HITRAN data locally for later reuse.")
                }
            }
        }

        Rectangle {
            id: separator3Rect
            Layout.fillWidth: true
            height: 1
            Layout.margins: 0
            visible: ((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? true : false
            color: "white"
        }


        // Such conditional implementation was introduced to make sure the fonts look properly both under desktop as well as the mobile platforms
        // Need to test this more thoroughly under mobile platforms, not sure all platforms will result in descent looks
        AuxButton {
            id: downloadHitranButton
            Layout.fillWidth: true
            platformText: qsTr("Download HITRAN")
            platformColor: 'green'

            // Enabling the closing of the form when the exit button is pressed
            MouseArea {
                anchors.fill: parent
                onClicked: {

                    hitranDownloadOptionsDataObj.downloadCompleteHitran();

                    var component = Qt.createComponent("Notifier.qml")
                    if (component.status === Component.Ready)
                    {
                        downloadNotifierWindow = component.createObject(hitranDownloadOptionsItem)
                    }
                    else
                    {
                        messageDialog.show(component.errorString())
                    }
                    downloadNotifierWindow.show()
                }
            }
        }
    }
}
