// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | info@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component containing elements to set the instrumental function parameters (used to smooth the spectra to emulate spectra recorded with an instrument)
import QtQuick 2.0
import QtQuick.Controls 1.3
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1

Item {

    id: instrumentFunctionItem
    anchors.fill: parent

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: "black"
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.right: instrumentFunctionColumnLayout.right
        anchors.top: instrumentFunctionColumnLayout.top
        anchors.left: instrumentFunctionColumnLayout.left
        anchors.bottom: instrumentFunctionColumnLayout.bottom
        anchors.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? commentLabel.height * (-0.85) : -20
        color: "white"
        border.width: 1
        border.color: "black"
        radius: height * 0.03
    }

    // Button to close the form
    BackRectangle {
        anchors.fill: exitButton
        anchors.margins: 8
    }
    ControlButton {
        anchors.right: instrumentFunctionItem.right
        anchors.top: instrumentFunctionItem.top
        id: exitButton
        platformText: qsTr("X")
        opacity: 1.0

        // Enabling the closing of the form when the exit button is pressed
        MouseArea
        {
            anchors.fill: parent
            onClicked: {
                specResultsDataObj.updatePlotAllPreserveScale()
                instrumentFunctionItem.destroy();
            }
        }
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: instrumentFunctionColumnLayout
        anchors.centerIn: parent
        spacing: 5
        transformOrigin: Item.Center
        Layout.fillHeight: true

        PlatformLabel {
            id: commentLabel
            text: qsTr("Specify the instrument function to use:")
        }

        GridLayout {

            // Instrument function combobox label
            PlatformLabel {
                // Layout.fillWidth: true
                anchors.right: instrumentalFunctionComboBox.left
                id: instrumentFunctionUnitsLabel
                Layout.column: 0
                Layout.row: 0
                text: qsTr("Instrumental   \n       function:   ")
            }

            // Instrumental function
            // 0 - None
            // 1 - Rectangular (Boxcar)
            // 2 - Triangular
            // 3 - Gaussian
            // 4 - Dispersion (Lorentz)
            // 5 - Diffraction
            // 6 - Michelson
            // ComboBox to select the instrumental function
            PlatformComboBox {
                id: instrumentalFunctionComboBox
                Layout.fillWidth: true
                Layout.column: 1
                Layout.row: 0
                model: [ "None", "Rectangular (Boxcar)", "Triangular", "Gaussian", "Dispersion (Lorentz)", "Diffraction", "Michelson"]
                currentIndex: instrFunctDataObj.instrFunctIndex
                onCurrentIndexChanged:  instrFunctDataObj.instrFunctIndex = currentIndex
                implicitWidth: commentLabel.width * 0.5
                style: ((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? winReferenceComboBox.style : referenceComboBox.style
            }
            // This weird way was introduced to enable text black color under the windows phone platform while maintaining the style a default for other platforms
            PlatformComboBox {
                id: referenceComboBox
                visible: false
            }
            PlatformComboBox {
                id: winReferenceComboBox
                style: ComboBoxPlatformStyle {}
                visible: false
            }

            // Label for the instrument function resolution, cm-1
            PlatformLabel {
                //Layout.fillWidth: true
                anchors.right: instrResolTextField.left
                id: instrResolLabel
                Layout.column: 0
                Layout.row: 1
                text: qsTr("         Instrumental  \nresolution, g, (cm-1):   ");
            }

            // Text field to specifty the instrumental resolution, cm-1
            PlatformTextField {
                Layout.fillWidth: true
                id: instrResolTextField
                Layout.column: 1
                Layout.row: 1
                text: instrFunctDataObj.instrResol
                onTextChanged: instrFunctDataObj.instrResol = text
                implicitWidth: unitsComboBox.width
            }

            // Label for the instrument function wing, cm-1
            SmallerFontLabel {
                // Layout.fillWidth: true
                anchors.right: instrFunctWingTextField.left
                Layout.column: 0
                Layout.row: 2
                text: qsTr("Instrumental function   \n               wing, (cm-1):");
            }

            // Text field to specify the instrumental function wing, cm-1
            PlatformTextField {
                Layout.fillWidth: true
                id: instrFunctWingTextField
                Layout.column: 1
                Layout.row: 2
                text: instrFunctDataObj.instrFunctWing
                onTextChanged: instrFunctDataObj.instrFunctWing = text
                implicitWidth: unitsComboBox.width
            }
        }

        // This portion of code is to display to the user what the instrument resolution is for different instrument functions selected
        Rectangle {
            id: separator2Rect
            Layout.fillWidth: true
            height: 1
            Layout.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows')) ? 2 : 10
            color: "black"
            visible: {
                if ((instrFunctDataObj.instrFunctIndex == 0) || (instrFunctDataObj.instrFunctIndex == 5)) { return false; }
                else { return true; }
            }
        }
        PlatformLabel {
            Layout.fillWidth: true
            anchors.right: instrResolTextField.left
            id: instrResolCommentLabel
            Layout.column: 0
            Layout.row: 1
            text: {
                if (instrFunctDataObj.instrFunctIndex == 0) { return qsTr(""); }
                else if (instrFunctDataObj.instrFunctIndex == 1) { return qsTr("g - slit width or the instrument resolution"); }
                else if (instrFunctDataObj.instrFunctIndex == 2) { return qsTr("g - line width == half base of the triangle"); }
                else if (instrFunctDataObj.instrFunctIndex == 3) { return qsTr("g/2 - Gaussian half-width at half-maximum"); }
                else if (instrFunctDataObj.instrFunctIndex == 4) { return qsTr("g/2 - Lorentzian half-width at half-maximum"); }
                else if (instrFunctDataObj.instrFunctIndex == 5) { return qsTr(""); }
                else if (instrFunctDataObj.instrFunctIndex == 6) { return qsTr("1/g - the maximum optical path difference"); }
            }
            visible: {
                if ((instrFunctDataObj.instrFunctIndex == 0) || (instrFunctDataObj.instrFunctIndex == 5)) { return false; }
                else { return true; }
            }
        }
    }
}
