// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | info@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component to change molecules parameters
import QtQuick 2.4
import QtQuick.Controls 1.3
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1

Item {

    id: moleculesItem
    anchors.fill: parent

    // Used to rotate the menu
    onHeightChanged:
    {
        if (Screen.primaryOrientation == Qt.PortraitOrientation)
        {
            moleculesColumnLayout.height = moleculesItem.height - exitButton.height * 2.5
        }
        else
        {
            moleculesColumnLayout.height = moleculesItem.height - exitButton.height * 0.7
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: backRect
        anchors.fill: parent
        border.width: 1
        border.color: "black"
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: frameRect
        anchors.right: moleculesColumnLayout.right
        anchors.top: moleculesColumnLayout.top
        anchors.left: moleculesColumnLayout.left
        anchors.bottom: moleculesColumnLayout.bottom
        anchors.margins: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? titleLabel.height * (-0.425) : -20
        anchors.topMargin: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'android') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? titleLabel.height * (-0.35) : -20
        color: "white"
        border.width: 1
        border.color: "black"
        radius: height * 0.02
    }

    Text {
        id: molWidthText
        text: "Parts per million, ppmv"
        visible: false
        enabled: false
    }

    // Column layout containing controls
    ColumnLayout {

        id: moleculesColumnLayout
        anchors.centerIn: parent
        height: moleculesItem.height - exitButton.height * 2.5

        // Label containing the title for the current window
        PlatformLabel {
            id: titleLabel
            anchors.top: moleculesColumnLayout.top
            text: "  Concentration may be specified in %, parts    \n  per million (ppmv), or as partial pressure    "
        }

        // Grid layout to contain information for all molecules (selected state, chemical formula, color to be used in the plot)
        // This list has to be populated during the program execution depending on the data in the configuration files or the internal information
        RowLayout {
            id: moleculesTitlesRowLayout
            anchors.top: titleLabel.bottom
            anchors.topMargin: 5
            Layout.fillWidth: true

            PlatformLabel {
                id: indexTitleLabel
                text: "  X"
            }

            // Label for the molecule name and checked state
            // Currently the program does not monitor the selection of all moleulces manually (if all are
            // unchecked or checked manually) then the select/deselect all will not update, should be taken into account later
            PlatformCheckBox {
                id: moleculeAllCheckboxTitle
                platformText: " Select All"
                checked: MolListViewModel.allSelected  // Need to change this to take into account if all are selected by default (currently they are)
                onClicked: {
                    MolListViewModel.selectAll(checked)
                }
                style: !((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? referenceCheckBox.style : 0
            }
            // This weird way was introduced to enable text black color under the windows phone platform while maintaining the style a default for other platforms
            CheckBox {
                id: referenceCheckBox
                visible: false
            }

            // Combobox for the concentration units
            PlatformComboBox {
                id: unitsComboBox
                model: [  "Concentration, %", "Parts per million, ppmv", "Atmospheres, atm", "Millibar, mbar", "Torr, Torr", "Pascal, Pa", "Pounds per square inch, psi"]
                currentIndex:  MolListViewModel.units
                implicitWidth: (Qt.platform.os == 'android') ? molWidthText.width * 1.06 : molWidthText.width * 1.13
                onCurrentIndexChanged:
                {
                    MolListViewModel.units = currentIndex
                    totalPressObj.updateTotalPress()
                }
                style: ((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? winReferenceComboBox.style : referenceComboBox.style
            }
            // This weird way was introduced to enable text black color under the windows phone platform while maintaining the style a default for other platforms
            PlatformComboBox {
                id: referenceComboBox
                visible: false
            }
            PlatformComboBox {
                id: winReferenceComboBox
                style: ComboBoxPlatformStyle {}
                visible: false
            }

            PlatformLabel {
                id: afterTitleLabel
                text: " "
            }
        }

        // Flickable is used to add a scroll bar when the number of items is larger than the screen can fit
        ScrollView {
            id: molScrollView
            anchors.top: moleculesTitlesRowLayout.bottom
            anchors.topMargin: 5
            anchors.right: moleculesColumnLayout.right
            anchors.left: moleculesColumnLayout.left
            anchors.bottom: moleculesColumnLayout.bottom

            ListView {
                id: molListView
                anchors.fill: parent
                Binding on spacing {
                    when: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone'))
                    value: unitsComboBox.height * 0.1
                }
                model: MolListViewModel

                delegate: RowLayout {

                    id: moleculeDelegate
                    implicitHeight: molSelNameCheckBox.implicitHeight

                    // Was done in such manner to take into accout different widths of labels under Mac (other platforms appear to be fine)
                    Rectangle
                    {
                        anchors.top: moleculeDelegate.top
                        anchors.left: moleculeDelegate.left
                        anchors.bottom: moleculeDelegate.bottom

                        id: indexRect
                        PlatformLabel {
                            anchors.centerIn: parent
                            id: indexLabel
                            text: model.index
                        }
                        implicitWidth: indexTitleLabel.width
                        implicitHeight: indexTitleLabel.height
                    }

                    PlatformCheckBox {
                        id: molSelNameCheckBox
                        // Had to do this here to overcome a reported bug
                        // http://stackoverflow.com/questions/20552647/changing-a-model-of-a-qml-listview-doesnt-change-the-corresponding-delegate
                        Binding {
                            target: molSelNameCheckBox
                            property: 'checked'
                            value: model.checked
                        }
                        platformText: (Qt.platform.os == 'ios') ? model.molName : "  " + model.molName
                        implicitWidth: moleculeAllCheckboxTitle.width
                        onCheckedStateChanged: model.checked = checked
                        style: !((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? referenceCheckBox.style : 0
                    }

                    // Implement platform specific resizing here later for the form to look better

                    // Text field to change the partial pressure or concentration for the current molecule
                    PlatformTextField {
                        id: molConcentration
                        text: model.pressValue
                        implicitWidth:  molListView.width - indexTitleLabel.width - moleculeAllCheckboxTitle.width - colorRect.width - 5 * moleculeDelegate.spacing // unitsComboBox.width * 0.76
                        onTextChanged: model.pressValue = text   // Used to update the value of the partial pressure or concentration in the model
                    }

                    // Rectanlge to contain the color corresponding to the current molecule to be used for plotting
                    Rectangle {
                        id: colorRect
                        color: model.color
                        implicitWidth: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows')) ? moleculeDelegate.height * 0.95 : moleculeDelegate.height * 0.85
                        implicitHeight: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'windows')) ? moleculeDelegate.height * 0.95 : moleculeDelegate.height * 0.85
                        radius: 5

                        // Enabling the closing of the form when the exit button is pressed
                        MouseArea {
                            anchors.fill: parent
                            onClicked:
                            {
                                MolListViewModel.selectedColor = model.color
                                MolListViewModel.activeIndex = model.index
                                var component = Qt.createComponent("ColorDialog.qml")
                                if (component.status === Component.Ready)
                                {
                                    colorDialogWindow = component.createObject(moleculesItem)
                                }
                                else
                                {
                                    messageDialog.show(component.errorString())
                                }
                                colorDialogWindow.show()
                            }
                        }
                    }

                    // Maybe need this for iOS for better looks
                    // Label {
                    //      id: afterListDelegateLabel
                    //      text: " "
                    // }
                }
            }
        }
    }

    // Button to close the form
    BackRectangle {
        anchors.fill: exitButton
        anchors.margins: 8
    }
    ControlButton {
        anchors.right: moleculesItem.right
        anchors.top: moleculesItem.top
        id: exitButton
        platformText: qsTr("X")
        opacity: 1.0

        // Enabling the closing of the form when the exit button is pressed
        MouseArea {
            anchors.fill: parent
            onClicked: {
                molListViewReducedModel.updateDisplayed();
                specResultsDataObj.updatePlotAllPreserveScale();
                moleculesItem.destroy();
            }
        }
    }
}
