// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | info@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// Splash screen functionality was implemented using random posts online including the ones as follows:
// https://medium.com/@benlaud/complete-guide-to-make-a-splash-screen-for-your-qml-android-application-567ca3bc70af#.rsap1sclk
// https://forum.qt.io/topic/38925/qt-on-android-working-example-of-a-splash-screen/3
// and the links on the obove pages
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML file corresponding to the main window ui file.  Contains the plot and the main navigation items.
import QtQuick 2.4
import QtQuick.Controls 1.3
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1

ApplicationWindow {

    // Need to prevent the possible repeat open of dialogs

    id: mainWindow
    title: qsTr("bytran -|- spectral calculations for portable devices using the HITRAN database")

    // This is for desktop only
    width: 800
    height: 480
    visible: true

    // Handing the case when the user presses the hardware back button to initiate the program close with a dialog
    onClosing: {
        close.accepted = false
        exitDialog.show();
    }

    MainForm {

        anchors.fill: parent        

        spectrumPlot.onHeightChanged: {
            // Saving the height of the screen to get the vertical resolution for ploting
            specResultsDataObj.heightPoints = spectrumPlot.xHeight
            specResultsDataObj.updatePlotMolsOnlyPreserveScale()
        }

        // Passing the height of the screen to the spectral data object to avoid plotting spectra with no data or very weak lines
        onHeightChanged: {
            // This had to be initialized here for the heigt of the item to be correct
            totalsButtonRect.height = menuButtonText.height * 1.9

            // Notification window is displayed when the height of the window is changed
            // Need to replace this with some kind of an on-load signal
            var component = Qt.createComponent("Notifier.qml")
            if (component.status === Component.Ready)
            {
                notifierWindow = component.createObject(mainWindow)
            }
            else
            {
                messageDialog.show(component.errorString())
            }
            notifierWindow.show()
        }

        // Transfering the pinch gesture coordinates to the plot
        plotPinchArea.onPinchStarted: {
            spectrumPlot.touchCenterX =  pinch.center.x;
            spectrumPlot.touchCenterY = pinch.center.y;
        }
        plotPinchArea.onPinchUpdated: {
            spectrumPlot.scale = pinch.scale; }

        totalMolsButton.onCheckedChanged: {
            showTotalOptionsRectangle.visible = totalMolsButton.checked; }

        // Opening the main menu
        menuButton.onClicked: {
            var component = Qt.createComponent("Menu.qml")
            if (component.status === Component.Ready)
            {                
                menuWindow = component.createObject(mainWindow)                
            }
            else
            {
                messageDialog.show(component.errorString())
            }
            menuWindow.show()
        }

        // Running the calculation
        calcButton.onClicked: {
            specResultsDataObj.showPlainTotal = true;
            specResultsDataObj.showInstrTotal = true;
            coreObj.calculate();
        }
        exitButton.onClicked: {

            if ((Qt.platform.os == 'ios'))
            {
                var component = Qt.createComponent("ExitDialog.qml")
                if (component.status === Component.Ready)
                {
                    exitWindow = component.createObject(mainWindow)
                }
                else
                {
                    messageDialog.show(component.errorString())
                }
                exitWindow.show()
            }
            else
            {
                exitDialog.show()
            }
        }

        // Opening a window to change the molecular parameters
        moleculesButton.onClicked: {
            var component = Qt.createComponent("Molecules.qml")
            if (component.status === Component.Ready)
            {
                moleculesWindow = component.createObject(mainWindow)

            }
            else
            {
                messageDialog.show(component.errorString())
            }
            moleculesWindow.show()
        }

        // This section is to save the plot to a file
        spectrumPlot.onSaveImageChanged: {
            // This is done in order to add the information string at the top of the plot
            totalsButtonRect.height = 0
            spectrumPlot.addTopString()
            plotAndMolViewRect.grabToImage(function(result) { result.saveToFile(spectrumPlot.saveImage); });
        }
        spectrumPlot.onFileWritingCompleteChanged: {
            shareResultsDataObj.shareImageAndText();
            spectrumPlot.updatePlot()
            notifyObj.notification = ''
            notifyObj.visible = false
            totalsButtonRect.height = totalMolsButton.height * 1.15
            shareResultsDataObj.shareResults = false
        }
    }

    // General purpose dialog
    MessageDialog {
        id: messageDialog
        title: qsTr("Would you like to perform this action?")
        function show(caption) {
            messageDialog.text = caption;
            messageDialog.open();
        }
    }

    // Dialog to exit an applicaion
    MessageDialog {
        id: exitDialog
        icon: StandardIcon.Question
        standardButtons: StandardButton.Yes | StandardButton.No

        function show() {
            exitDialog.text = "Would you like to exit the application?";
            exitDialog.open()
        }
        onYes: { coreObj.saveConfiguration(); Qt.quit(); }
        onNo: { exitDialog.close(); }
    }   
}
